`timescale 1ns / 100ps

`define MEM_SIZE 'h40_0000 /* 16-bit words */

module main();
	reg clk;
	wire [2:0]	 fc;
	wire [23:1] 	 addr;
	wire 		 _as;
	wire 		 _uds;
	wire 		 _lds;
	wire 		 r_w;
	reg 		 _dtack;
	wire [15:0] 	 data;
	reg [2:0] 	 _ipl;
	reg 		 _avec;
	reg 		 _br;
	wire 		 _bg;
	reg 		 _berr;
	reg 		 _reset;
	reg 		 _halt;
	reg [15:0] 	 mem[0:`MEM_SIZE-1];	 

	localparam bus_idle	= 2'b00;
	localparam bus_read	= 2'b01;
	localparam bus_write	= 2'b10;
	localparam bus_finish	= 2'b11;
	reg [1:0] 	bus_state;
	reg		dout;       

	m68k cpu(clk, fc, addr, _as, _uds, _lds, r_w, _dtack, data,
		 _ipl, _avec, _br, _bg, _berr, _reset, _halt);

        defparam cpu.turbo = 1;
//	defparam cpu.log_flags = 0;
	defparam cpu.log_flags = 'hff;
	
	always@(posedge clk or negedge _reset) begin
		if (_reset == 0) begin
			bus_state <= bus_idle;
			dout <= 0;			
		end else begin
			case (bus_state)
				/* S3->S4, can latch addr and R/W */
				/* we assert DACK_ immediately because we
				 * know we'll deal with the data in the next
				 * cycle and we don't do bus errors yet
				 */				 
				bus_idle: begin
					dout <= 0;
					if (_as == 0) begin
						bus_state <= r_w ? bus_read : bus_write;
						_dtack <= 0;
					end
				end
				/* S5 -> S6, present or latch data */
				bus_read: begin
					bus_state <= bus_finish;
					dout <= 1;
				end
				/* S5 -> S6, present or latch data */
				bus_write: begin
					bus_state <= bus_finish;
					mem[addr] <= data;					
				end
				bus_finish: begin
					_dtack <= 1;
					bus_state <= bus_idle;
				end
			endcase // case (bus_state)
		end // else: !if(reset_ == 0)
	end	

	assign data = (dout) ? mem[addr] : 16'bz;

	initial begin
                $dumpfile("sim_bench.vcd");
                $dumpvars(0,cpu);

		clk = 0;
		_reset = 0;
		_halt = 1;
		_berr = 1;
		_br = 1;
		_ipl = 3'b111;
		_avec = 1;
		_dtack = 1;		

		mem[0] = 0;
		mem[1] = 'h2000 - 16;
		mem[2] = 0;
		mem[3] = 'h1000;
		// 1000:       303c 000a       movew #10,%d0	
		mem['h1000/2] = 'h303c;
		mem['h1002/2] = 'h000a;
		// 1004:       41fa 001a       lea %pc@(1020 <_buf>),%a0
		mem['h1004/2] = 'h41fa;
		mem['h1006/2] = 'h001a;				    
		// 1008:       3080            movew %d0,%a0@
		mem['h1008/2] = 'h3080;				    
		// 100a:       0640 0001       addiw #1,%d0
		mem['h100a/2] = 'h0640;
		mem['h100c/2] = 'h0001;
		// 100e:       6000 fff4       braw 1004
		mem['h100e/2] = 'h6000;
		mem['h1010/2] = 'hfff4;
 
		#1000 _reset = 1;		
		#10000000 $finish;		
	end

	always begin
		/* Simulate 5Mhz */
		#100 clk = ~clk;
	end

endmodule 